@@ -1,3 +1,9 @@
+#!/usr/bin/env python
+# coding: utf-8
+
+# In[1]:
+
+
 ############################### Paquetes ##############################################
 import pandas as pd
 from scipy import stats
@@ -84,6 +90,21 @@ def Selec_Comuna(D1,cve): #(D1 es la tabla, cve: es el codigo de la comuna)
     D_comuna=D1.loc[(D1.loc[:,'cve_comuna']==cve) & (D1.loc[:,'valor']>=tol)]
     return D_comuna
 
+def etiqueta(mini,maxi):
+    etiquetas=[]
+    en='Index_antiguedad_10_'
+    L=["A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","W","X","Y","Z"]
+    k=0
+    for j in range(mini,maxi,10):
+        if j<maxi-10:
+            p=L[k]+str(j)[2:]+'/'+str(j+9)[2:]
+            etiquetas.append(en+p)
+        if j>maxi-10:
+            p=L[k]+str(j)[2:]+'/'+str(maxi)[2:]
+            etiquetas.append(en+p)
+        k+=1
+    return etiquetas
+
 # Eliminación de datos duplicados
 def datosduplicados(tabla,T): #(tabla es la tabla, T es True entonces se muestra la cantidad de datos eliminados)
     n_inicial = tabla.shape[0];
@@ -463,15 +484,66 @@ def graficosns(df,columna,option1,option2,binss,colore):   #(df:tabla, columna,
     ax[0].grid(True, color='lightgrey',linestyle='--')
     ax[0].set(xlabel='')
     ax[0].set(ylabel='')
+    
+
     sns.histplot(data=df,x=columna,hue='Tipo',element='bars',palette=colore,
                  ax=ax[1],kde=option1,discrete=option2,bins=binss)     
     ax[1].set(xlabel='')
     ax[1].set(ylabel='Cantidad')
     #ax[1].set_title(columna)
     ax[1].grid(True, color='lightgrey',linestyle='--')
-    plt.show()   
+    plt.show()
+    
+def sub_clusters(df):
+    # Hacemos una lista con las etiquetas de los clusters
+    L_C=df['cluster_elbow'].unique()
+    L_C=L_C.tolist()
+    L_C.sort()
+    # lista para guardar cada data frame nuevo:
+    List_data=[]
+    # Empezamos a realizar el k-means a cada cluster:
+    for P in L_C:
+        df_p=df[df['cluster_elbow']==P]
+        df_P=df_p.drop(['cluster_elbow'],axis=1)
+        X = StandardScaler().fit_transform(df_P)
+        kmeans_kwargs = {"init": "k-means++","n_init": 10,"max_iter": 300,"random_state": 42,}
+        sse = []
+        for k in range(1, 11):
+            kmeans = KMeans(n_clusters=k, **kmeans_kwargs)
+            kmeans.fit(X)
+            sse.append(kmeans.inertia_)
 
-def indexacion(df,mini,maxi):
+        fig = plt.figure(figsize=(3,3))
+        plt.plot(range(1, 11),sse,color="blue",marker='o')
+        plt.xticks(range(1, 11))
+        plt.xlabel("Número de Clusters")
+        plt.ylabel("SSE")
+        plt.show()
+        kl = KneeLocator(range(1, 11), sse, curve="convex", direction="decreasing")
+        el=kl.elbow
+        print(f"El número de clusters apropiados es: {el}")
+        kmeanss=KMeans(n_clusters=el,**kmeans_kwargs)
+        kmeanss.fit(X)
+        df_P.loc[:,'subcluster']=kmeanss.labels_
+        df_P.insert(df_P.shape[1]-1,'cluster_elbow',df_p.loc[:,'cluster_elbow'], True)
+        List_data.append(df_P)
+        display(df_P)
+        fig = plt.figure(figsize=(6,4))
+        sns.scatterplot(x='longitud',y='latitud',data=df_P,hue="subcluster",palette="Set2",edgecolor="black",linewidth=0.3)
+        plt.legend(labels=((df_P['subcluster'].unique()).tolist()).sort(),
+                   bbox_to_anchor=(1.02, 1), loc='upper left', borderaxespad=0)
+        plt.show()
+    DF_P=pd.concat(List_data, ignore_index=True, sort=False)
+    C_e=DF_P.loc[:,'cluster_elbow'].tolist()
+    S_c=DF_P.loc[:,'subcluster'].tolist()
+    L_CF=[]
+    for num in range(len(C_e)):
+        L_CF.append(str(C_e[num])+'.'+str(S_c[num]))
+    DF_P.insert(DF_P.shape[1],'cluster',L_CF, True)
+    return DF_P
+
+def indexacion(df1,mini,maxi):
+    df=df1.copy()
     years1=[]
     for i in range(mini,maxi,10):
         years1.append(i)
@@ -582,8 +654,8 @@ def Escalas(Xrlinter_train, Xrlinter_test):
 
 
 ## Random Forest para un par de datos
-def RForest(Xtrain,Xtest,ytrain,ytest,k):
-    rforest = RandomForestRegressor(max_features=7,random_state=42)  
+def RForest(Xtrain,Xtest,ytrain,ytest,k,maxfeat):
+    rforest = RandomForestRegressor(max_features=maxfeat,random_state=42)  
     params={'n_estimators':[300,500,800],
            'max_depth':[30,50,80]}  
     Search=GridSearchCV(estimator=rforest,
@@ -594,14 +666,14 @@ def RForest(Xtrain,Xtest,ytrain,ytest,k):
     y_pred43 = cross_val_predict(rforest1, Xtest, np.ravel(ytest), cv=k)
     return rforest1,y_pred43
 
-def RForestEleccion(df2,k):
+def RForestEleccion(df2,k,maxfeat):
     # Entrenamiento
     Xtodos_train, Xtodos_test, ytodos_train, ytodos_test,Xtodos=Entrenamiento(df2,0.2)
     # Escala
     Xtodos_train, Xtodos_test,scl1 =Escalas(Xtodos_train, Xtodos_test)
     # Random Forest
-    rforest1,y_pred43=RForest(Xtodos_train,Xtodos_test,ytodos_train,ytodos_test,k)
-    return scl1,ytodos_test,y_pred43,rforest1
+    rforest1,y_pred43=RForest(Xtodos_train,Xtodos_test,ytodos_train,ytodos_test,k,maxfeat)
+    return scl1,ytodos_test,y_pred43,rforest1,Xtodos_train
 
 # XGB para un par de datos
 def XGB(Xtrain,Xtest,ytrain,ytest,k):
@@ -630,7 +702,7 @@ def XGB_Eleccion(df2,k):
     #print("El método de Extreme Gradient Boosting se ha ejecutado con éxito. Se procederá a guardar los resultados.")
     #joblib.dump(xgb1,"XGB_"+str(cve)+".joblib")
     # (Se hace aparte)
-    return scl1,ytodos_test,y_pred43,xgb1
+    return scl1,ytodos_test,y_pred43,xgb1,Xtodos_train
 
 # Error para los 4 modelos
 def porcentajeerror2(e,f,g,h):  ##### MODIFICADA POR TENER RL Y ENET #####
@@ -867,7 +939,7 @@ def Tabla_Estadistica_Error2(df): ## MOD X TENER RL Y ENET ##
         tabla=pd.concat([tabla, tablaux],ignore_index=True);
     return tabla
 
-def ModeloFinal(reml2,scls,models,cve): ### MOD X TENER RL Y ENET
+def ModeloFinal(reml2,scls,models,cve,Malerta1,Malerta2): ### MOD X TENER RL Y ENET
     RF_er=np.array(list(map(abs, reml2['% de error RF'].tolist())))
     XGB_er=np.array(list(map(abs, reml2['% de error XGB'].tolist())))
     L=[RF_er,XGB_er] #Lista 
@@ -882,11 +954,14 @@ def ModeloFinal(reml2,scls,models,cve): ### MOD X TENER RL Y ENET
         scaler_file = "escala_Final_"+str(cve)+".save"
         joblib.dump(scls[0], scaler_file)
         joblib.dump(models[0],"Modelo_Final_"+str(cve)+".joblib")
-        return models[0],scls[0]
+        Malerta1.to_excel("Muestra_Alertas_"+str(cve))
+        return models[0],scls[0],Malerta1
     
     elif p==a4: 
         print("El método escogido de manera definitiva será el Extreme Gradient Boosting. Se procede a guardarlo")
         scaler_file = "escala_Final_"+str(cve)+".save"
         joblib.dump(scls[1], scaler_file)
         joblib.dump(models[1],"Modelo_Final_"+str(cve)+".joblib")
-        return models[1],scls[1]
\ No newline at end of file
+        Malerta2.to_excel("Muestra_Alertas_"+str(cve))
+        return models[1],scls[1],Malerta2
+
